from cleo.application import Application
from cleo.commands.list_command import ListCommand as HelpListCommand
from wgm import __version__
from wgm.commands.create import CreateCommand
from wgm.commands.delete import DeleteCommand
from wgm.commands.list import ListCommand
from wgm.commands.version import VersionCommand


def create_app():
    app = Application('WireGuard Manager', __version__)

    # Reconfigure default command list to help-list
    help_list_command = HelpListCommand()
    help_list_command.name = 'help-list'
    app._default_command = help_list_command.name
    app.add(help_list_command)

    app.add(CreateCommand())
    app.add(DeleteCommand())
    app.add(ListCommand())
    app.add(VersionCommand())

    return app


def main():
    app = create_app()
    app.run()


if __name__ == '__main__':
    main()



import os

from wgm.command import Command
from cleo.helpers import argument, option


class CreateCommand(Command):
    name = 'create'
    description = 'Create a new WireGuard user.'
    arguments = [
        argument(
            name='name',
            description='The user name',
        ),
    ]
    options = [
        option(
            'private-cdir',
            description=('Force private network to set user address'),
            default=False,
            flag=False,
        ),
        option(
            'allowed-cdirs',
            description=(
                'Set cdirs to be allowed to connect seprarated by comma, by '
                'default is 0.0.0.0/24'),
            default='0.0.0.0/24',
            flag=False,
        )
    ]

    def handle(self):
        self.read_config()
        name = self.argument('name')
        if name in self.config.users.keys():
            self.line(f'User "{name}" already exists.', 'error')
            return 1
        clients_path = os.path.join(self.config.config_path, 'clients')
        if not os.path.exists(clients_path):
            os.mkdir(clients_path)
        user_path = os.path.join(clients_path, name)
        if os.path.exists(user_path):
            self.line(f'User "{name}" already exists.', 'error')
            return 1
        os.mkdir(user_path)

        # Generate private key
        self.shell(
            f'wg genkey | tee {user_path}/private_key '
            f'| wg pubkey > {user_path}/public_key')
        user_private_key = self.read_file(f'{user_path}/private_key')
        user_public_key = self.read_file(f'{user_path}/private_key')

        # Prepare data
        user_ids = [int(u['id']) for u in self.config.users]
        user_id = 1
        while user_id not in user_ids and user_id != self.config.server_id:
            user_id += 1
        allowed_cdirs = (
            self.option('allowed-cdirs').split(',')
            if self.option('allowed-cdirs') else []
        )
        allowed_cdirs.insert(0, self.dict_to_cdir(self.config.cdir))
        user_address = self.config.cdir
        user_address['ip'][3] = user_id
        user_address['mask'] = 32

        # Create config user
        self.write_config_file(f'{user_path}/{name}.conf', {
            'Interface': [{
                'PrivateKey': user_private_key,
                'Address': self.dict_to_cdir(user_address),
            }],
            'Peer': [{
                'PublicKey': self.config.server_public_key,
                'AllowedIPs': ', '.join(allowed_cdirs),
                'Endpoint': self.config.endpoint,
                'PersistentKeepalive': 25,
            }]
        })
        self.line(f'Config file "{user_path}/{name}.conf" created.', 'info')

        # Update server config
        if 'Peer' not in self.config.wg:
            self.config.wg['Peer'] = []
        if not isinstance(self.config.wg['Peer'], list):
            self.config.wg['Peer'] = [self.config.wg['Peer']]
        self.config.wg['Peer'].append({
            'PublicKey': user_public_key,
            'AllowedIPs': self.dict_to_cdir(user_address),
        })
        self.write_config_file(self.config.wg_config_file, self.config.wg)
        self.line(f'User "{name}" created.', 'success')



import os
import shutil

from wgm.command import Command
from cleo.helpers import argument


class DeleteCommand(Command):
    name = 'delete'
    description = 'Delete a WireGuard user.'
    arguments = [
        argument(
            name='name',
            description='The user name',
        ),
    ]

    def handle(self):
        self.read_config()
        name = self.argument('name')
        public_key = None
        for key, user in self.config.users.items():
            if user['name'] == name:
                public_key = key
                break
        if not public_key:
            self.line(f'User "{name}" not exists.', 'error')
            return 1
        for peer in self.config.wg['Peer']:
            if peer['PublicKey'] == public_key:
                self.config.wg['Peer'].remove(peer)
                break
        self.write_config_file(self.config.wg_config_file, self.config.wg)
        clients_path = os.path.join(self.config.config_path, 'clients')
        shutil.rmtree(os.path.join(clients_path, name))
        self.line(f'User "{name}" deleted.', 'success')



from wgm.command import Command


class ListCommand(Command):
    name = 'list'
    description = 'List all users.'

    def handle(self):
        self.read_config()
        users = self.config.users
        if not users:
            self.line('Not users.', 'warn')
            return 0
        table = self.table()
        table.set_headers(['ID', 'Name', 'Private key', 'Address'])
        table.set_rows([
            [
                str(user['id']),
                user['name'],
                private_key,
                self.dict_to_cdir(user['cdir']),
            ] for private_key, user in users.items()
        ])
        table.render()



import os
import subprocess

from cleo.commands.command import Command as BaseCommand
from cleo.helpers import option
from cleo.io.io import IO
from cleo.io.outputs.output import Verbosity
from wgm.config import Config


class Command(BaseCommand):

    def configure(self) -> None:
        self._definition.add_option(
            option(
                'config-path',
                'c',
                description='Set path to wireguard config',
                default='/etc/wireguard',
                flag=False,
            )
        )
        self._definition.add_option(
            option(
                'wg-config-file',
                'w',
                description='Set config file to wireguard config',
                default=False,
                flag=False,
            )
        )
        super().configure()

    def execute(self, io: IO) -> int:
        self._io = io
        self.config = Config(
            config_path=os.path.abspath(self.option('config-path')),
            wg_config_file=self.option('wg-config-file'),
        )
        if self.config.wg_config_file is False:
            self.config.wg_config_file = os.path.join(
                self.config.config_path, 'wg0.conf')

        self.add_style('error', fg='red', options=['bold'])
        self.add_style('info', fg='blue')
        self.add_style('warn', fg='yellow')
        self.add_style('success', fg='green')

        try:
            status_code = super().execute(io)
        except InterruptedError as exception:
            self.line(exception.strerror or 'Unknown error', 'error')
            status_code = exception.errno or 1
        return 0 if status_code is None else status_code

    def shell(self, cmd):
        self.line(f'$: {cmd}', 'info', verbosity=Verbosity.VERBOSE)
        process = subprocess.Popen(
            cmd, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE, text=True, shell=True)
        try:
            stdout, stderr = process.communicate(timeout=10)
            if stdout:
                self.line(stdout, 'success', verbosity=Verbosity.VERBOSE)
            if stderr:
                self.line(stderr, 'error', verbosity=Verbosity.VERBOSE)
        except subprocess.TimeoutExpired:
            process.kill()
            stdout, stderr = process.communicate()
            self.line_error(f'Timeout expired for command: {cmd}')
            raise Exception('Timeout expired')
        if process.returncode != 0 and stderr:
            self.line_error(f'Error executing command: {cmd}')
            self.line_error(stderr)
            raise Exception(process.returncode)
        return stdout.strip()

    def read_file(self, filename):
        if not os.path.exists(filename):
            return False
        with open(filename, 'r') as f:
            content = f.read()
        if content.endswith('\n'):
            content = content[:-1]
        return content

    def read_config_file(self, filename):
        config = {}
        current_section = None
        with open(filename, 'r') as file:
            for line in file:
                line = line.strip()
                if not line or line.startswith('#'):
                    continue
                if line.startswith('[') and line.endswith(']'):
                    current_section = line[1:-1]
                    if current_section not in config:
                        config[current_section] = {}
                    else:
                        if isinstance(config[current_section], list):
                            config[current_section].append({})
                        else:
                            config[current_section] = [
                                config[current_section], {}]
                elif current_section is None:
                    continue
                else:
                    key, value = line.split('=', 1)
                    key = key.strip()
                    value = value.strip()
                    section = config[current_section]
                    if isinstance(section, list):
                        section = section[-1]
                    if key in section:
                        if isinstance(section[key], list):
                            section[key].append(value)
                        else:
                            section[key] = [section[key], value]
                    else:
                        section[key] = value
        return config

    def write_config_file(self, filename, config_dict):
        with open(filename, 'w') as file:
            for section, items in config_dict.items():
                if isinstance(items, dict):
                    items = [items]
                for item in items:
                    file.write(f'[{section}]\n')
                    for key, value in item.items():
                        if isinstance(value, list):
                            for v in value:
                                file.write(f'{key} = {v}\n')
                        else:
                            file.write(f'{key} = {value}\n')
                    file.write('\n')

    def read_config(self):
        # Check paths
        if not os.path.exists(self.config.config_path):
            raise InterruptedError(
                100, f'Path {self.config.config_path} not exists!')
        if not os.path.exists(self.config.wg_config_file):
            raise InterruptedError(
                101, f'Config file {self.config.wg_config_file} not exists!')

        # Read users
        clients_path = os.path.join(self.config.config_path, 'clients')
        if not os.path.exists(clients_path):
            os.mkdir(clients_path)
        self.config.users = {}
        for file in os.listdir(clients_path):
            absfile = os.path.join(clients_path, file)
            if not os.path.isdir(absfile):
                continue
            if any(file.startswith(x) for x in ['.', '_']):
                continue
            conf_file = os.path.join(clients_path, file, f'{file}.conf')
            if not os.path.exists(conf_file):
                continue
            config = self.read_config_file(conf_file)
            public_key = self.read_file(
                os.path.join(clients_path, file, 'public_key'))
            cdir = self.cdir_to_dict(config['Interface']['Address'])
            self.config.users[public_key] = {
                'id': cdir['ip'][3],
                'name': file,
                'cdir': cdir,
                'config': config,
            }

        # Read wg0
        self.config.wg = self.read_config_file(self.config.wg_config_file)
        self.config.hostname = self.shell('hostname -f')
        self.config.endpoint = ':'.join([
            self.config.hostname,
            self.config.wg['Interface']['ListenPort']
        ])
        cdir = self.cdir_to_dict(self.config.wg['Interface']['Address'])
        self.config.server_id = cdir['ip'][3]
        self.config.cdir = cdir
        self.config.server_public_key = self.read_file(
            os.path.join(self.config.config_path, 'server_public_key')
        )

    def cdir_to_dict(self, cdir_address: str) -> dict:
        parts = cdir_address.split('/')
        ip = [int(b) for b in parts[0].split('.')]
        mask = int(parts[1]) if len(parts) == 2 else 24
        return {
            'ip': ip,
            'mask': mask,
        }

    def dict_to_cdir(self, cdir_address: dict) -> str:
        ip = '.'.join([str(b) for b in cdir_address['ip']])
        return f'{ip}/{cdir_address["mask"]}'




class Config:

    def __init__(self, **args):
        self.__dict__.update(args)

    def get(self, key, default=None):
        return getattr(self, key, default)

    def __str__(self) -> str:
        return str(self.__dict__)

    def __repr__(self) -> str:
        return self.__str__()



from wgm import __version__
from wgm.command import Command


class VersionCommand(Command):
    name = 'version'
    description = 'Show version.'
    check_config = False

    def handle(self):
        self.line(f'Version {__version__}', 'info')
